package com.example.demo.signapi;

import com.example.demo.signapi.client.api.DocumentApi;
import com.example.demo.signapi.client.api.SigningOrderApi;
import com.example.demo.signapi.client.invoker.ApiClient;
import com.example.demo.signapi.client.invoker.ApiException;
import com.example.demo.signapi.client.model.Document;
import com.example.demo.signapi.client.model.DocumentRefId;
import com.example.demo.signapi.client.model.Method;
import com.example.demo.signapi.client.model.SigningOrder;
import com.example.demo.signapi.client.model.SigningOrderRefId;
import com.example.demo.signapi.client.model.Task;
import com.example.demo.signapi.client.model.TaskStatusInfo;
import com.github.scribejava.core.builder.ServiceBuilder;
import com.github.scribejava.core.builder.api.DefaultApi20;
import com.github.scribejava.core.oauth.OAuth20Service;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.util.UUID;

public class App {
    public static void main(String[] args) throws Exception {

        // Get a valid access token
        String accessToken = getAccessToken();

        // Set up our ApiClient to use the access token
        ApiClient apiClient = new ApiClient();
        apiClient.setBasePath("https://preprod.signicat.com/sign/");
        apiClient.setAccessToken(accessToken);

        // Set up DocumentApi and SigningOrderApi to use the ApiClient
        DocumentApi documentApi = new DocumentApi(apiClient);
        SigningOrderApi signingOrderApi = new SigningOrderApi(apiClient);

        // Upload a document and get a reference ID to it
        String documentRefId = uploadDocument(documentApi);

        String taskId = UUID.randomUUID().toString();
        String orderId = createSigningOrder(signingOrderApi, documentRefId, taskId);

        String signingUrl = String.format("https://preprod.signicat.com/std/docaction/demo?request_id=%s&task_id=%s", orderId, taskId);

        System.out.println("Signing order created, sign the document at " + signingUrl);
        System.out.println("Waiting for signing to be completed");

        // Naively poll task status until the task has been completed.
        // In your real implementation, you can use URL notifications to have Signicat send a request to your system
        // when a task changes status instead of polling like this.
        TaskStatusInfo.TaskStatusEnum taskStatus = null;
        while (taskStatus != TaskStatusInfo.TaskStatusEnum.COMPLETED) {
            Thread.sleep(5000);
            taskStatus = signingOrderApi.getStatus(taskId, orderId).getTaskStatus();
            System.out.println("Status: " + taskStatus);
        }

        // Task has been completed, download the resulting LTV-SDO as result.xml
        File tmpResult = signingOrderApi.getResultDocument("doc-1", taskId, orderId);
        Files.copy(tmpResult.toPath(), new File("result.pdf").toPath(), StandardCopyOption.REPLACE_EXISTING);
        System.out.println("Downloaded result document as result.pdf");
    }

    private static String uploadDocument(DocumentApi documentApi) throws ApiException {
        // Read a local file to upload
        File file = new File("src/main/resources/document.pdf");

        // Use DocumentApi to perform HTTP POST to /documents endpoint
        DocumentRefId documentRefId = documentApi.uploadFileToSds(file);
        return documentRefId.getDocumentId();
    }

    private static String createSigningOrder(SigningOrderApi signingOrderApi, String documentRefId, String taskId) throws Exception {
        // Create signing order model
        SigningOrder signingOrder = new SigningOrder();

        Task task = new Task();
        task.setId(taskId);

        // Add the document we uploaded earlier as a Document to be signed in the Task
        Document document = new Document();
        document.setId("doc-1");
        document.setDocumentRef(documentRefId);
        document.setDescription("Test document");
        document.setSource(Document.SourceEnum.SESSION);
        document.setAction(Document.ActionEnum.SIGN);

        task.getDocuments().add(document);

        // Use Norwegian BankID as our signing method
        Method method = new Method();
        method.setName("nbid-sign");
        method.setType(Method.TypeEnum.SIGNED_STATEMENT);

        task.getSignatureMethods().add(method);

        signingOrder.getTasks().add(task);

        // POST to /orders and return the SigningOrder's ID
        SigningOrderRefId signingOrderRefId = signingOrderApi.createRequest(signingOrder);
        return signingOrderRefId.getId();
    }

    private static String getAccessToken() throws Exception {
        // Set up OAuth client using ScribeJava and fetch access token
        String clientId = "test.demo.microservices";
        String clientSecret = "BVn1suS2TkKCMWejcTmxgW-Dmuul0wKSZ59liKa0bW4";

        OAuth20Service service = new ServiceBuilder(clientId)
                .apiSecret(clientSecret)
                .defaultScope("client.signature")
                .build(new DefaultApi20() {

                    @Override
                    public String getAccessTokenEndpoint() {
                        return "https://preprod.signicat.com/oidc/token";
                    }

                    @Override
                    protected String getAuthorizationBaseUrl() {
                        return null;
                    }
                });

        return service.getAccessTokenClientCredentialsGrant().getAccessToken();
    }
}
